﻿'DI_145.ChannelArray map:
'ChannelArray(0-3) analog inputs 0-3
'ChannelArray(8) all digital inputs (all four bits)

Imports dataq.simple

Public Class frmDI_145Example
    Dim DataqDeviceArray() As DataqDevice 'array containing all connected devices
    Dim WithEvents DI_145 As DataqDevice 'this is the primary device we'll work with
    Dim CollectedData(11) As Double 'dimension the array that will hold acquired data
    Dim ChannelConfig(11) As String 'array to retain the channel configuration defined via the GUI

    Private Sub frmDI_145Example_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        'upon loading the application do some device discovery, connection, and housekeeping

        'Discover connected devices
        DataqDeviceArray = Discovery.DiscoverAllDevices

        'Use only the first device in the discovery array
        'Bail if the first arrary position is empty
        If DataqDeviceArray.Length = 0 Then
            MessageBox.Show("No compatible DATAQ Instruments devices found.", _
                            "Missing DATAQ Instruments hardware", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Error)
            End

            'get here if the first array position contains something
        Else
            DI_145 = DataqDeviceArray(0) 'assign DataqDevice DI_145 to the first device in the array and...
            DI_145.Connect() '...connect to it
        End If

        'display model-specific information
        lblModel.Text = DI_145.Model
        lblSerial.Text = DI_145.SerialNumber
        lblPort.Text = DI_145.ComPort

        'Bail if the connected device is not model DI-145
        If DI_145.Model <> "DI-145" Then
            MessageBox.Show("This program works only with DATAQ Instruments model DI-145", _
                           "Incompatible Device (" + DI_145.Model + ")", _
                           MessageBoxButtons.OK, _
                           MessageBoxIcon.Error)
            End
        End If

        'load up ChannelConfig list with a default analog channel configuration
        For Channel As Integer = 0 To 3
            ChannelConfig(Channel) = "On" 'Enable all four analog input channels
        Next
        'rbAin1 is the only one checked upon load so...
        cbAnalogConfig.SelectedIndex = 1 'Display "On" to be consistent

        'now define the default settings for the digital inputs
        ChannelConfig(8) = "Off" 'turn off both discrete input bits

        'display "Off" to be consistent for all digital inputs
        cbDin0_1.SelectedIndex = 0

        'default is to configure an analog channel so hide the digital config combo box
        cbDin0_1.Visible = False

        'everthing's good, so create an event handler that fires when new DI-145 data is available
        AddHandler DI_145.NewData, AddressOf GetDI145Data
    End Sub
    Private Sub GetDI145Data()
        'this is the event handler for DI-145 data. Get here when new data is available during a scan
        Dim Scans As Integer = DI_145.NumberOfScansAvailable
        Dim Channels As Short = DI_145.NumberOfChannelsEnabled
        Dim DI_145_Data(Scans * Channels - 1) As Double 'will hold all data for the scan
        Dim ResponseString As String = ""

        'Move data into temporary array
        DI_145.GetInterleavedScaledData(DI_145_Data, 0, Scans)
        'now move it to the list box
        For Row As Integer = 0 To Scans - 1
            For Column As Short = 0 To Channels - 1
                ResponseString = ResponseString + DI_145_Data(Column + (Row * Channels)).ToString("'+'00.00;'-'00.00")
                If Column <> Channels - 1 Then
                    ResponseString = ResponseString + ", " 'append a comma only if this is not the last displayed value
                End If
            Next
            UpdateList(ResponseString + vbCrLf) 'output to the list box
            ResponseString = "" 'reset ResponseString
        Next
    End Sub
    Private Sub btnStart_Click(sender As System.Object, e As System.EventArgs) Handles btnStart.Click
        'this sub configures the DI-145 scan list. 
        'DI_145.ChannelArray map:
        'ChannelArray(0-3) analog inputs 0-3 (or 1-4 for normal people as per the device sticker)
        'ChannelArray(8) all digital inputs (all four bits)

        Dim Range As New dataq.Range(Of Double) 'holds the FSR of the rate channel. Analog ranges are fixed at ±10 Vfs

        'first, some GUI housekeeping
        btnStart.Enabled = False
        btnStop.Enabled = True
        lbOutput.Items.Clear()
        lblSamplingRate.Text = ""
        lblRealTime.Text = ""
        cbAnalogConfig.Enabled = False
        cbDin0_1.Enabled = False
        rbAin1.Enabled = False
        rbAin2.Enabled = False
        rbAin3.Enabled = False
        rbAin4.Enabled = False
        rbDin0_1.Enabled = False
        'Configure the DI-145's scan list for analog channels
        'do the following for each of the four analog channels:
        For Channel As Integer = 0 To 3
            'first disable the channel
            DI_145.ChannelArray(Channel).Enabled = False

            'now configure the channel as defined by array ChannelConfig
            If ChannelConfig(Channel) <> "Off" Then
                'not OFF so must be on
                DI_145.ChannelArray(Channel).Enabled = True ' enable the channel for acq
            End If
        Next

        'now set up the digital inputs

        If ChannelConfig(8) <> "Off" Then
            'not OFF so must be on, so enable 
            DI_145.ChannelArray(8).Enabled = True
        End If

        'display the actual sampling rate per channel
        lblSamplingRate.Text = DI_145.SampleRatePerChannel.ToString("F3")

        DI_145.NewDataMinimum = 1 'set the number of scans to acquire before the NewData event fires
        DI_145.Start() 'start scanning
    End Sub
    Private Sub btnStop_Click(sender As System.Object, e As System.EventArgs) Handles btnStop.Click
        'stop button clicked so stop the DI-145 and perform some GUI housekeeping
        DI_145.Stop()
        'gui housekeeping stuff
        btnStart.Enabled = True
        btnStop.Enabled = False
        cbAnalogConfig.Enabled = False
        cbDin0_1.Enabled = False

        cbAnalogConfig.Enabled = True
        cbDin0_1.Enabled = True
        rbAin1.Enabled = True
        rbAin2.Enabled = True
        rbAin3.Enabled = True
        rbAin4.Enabled = True
        rbDin0_1.Enabled = True
    End Sub
    'UpdateList is called from a different thread than the main program, so we need to delegate
    Private Delegate Sub UpdateListDelegate(ByVal ResponseString As String) 'create new class UpdateListDelegate
    Private Sub UpdateList(ByVal ResponseString As String)
        If Me.InvokeRequired Then
            Me.Invoke(New UpdateListDelegate(AddressOf UpdateList), ResponseString) 'invoke delegate then...
        Else
            lbOutput.Items.Add(ResponseString) '...output string to list box
            lblRealTime.Text = ResponseString
        End If
    End Sub

    'the below is all mundane GUI management stuff, none of which accesses the DI-145 directly. 
    Private Sub cbChannelConfig_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbAnalogConfig.SelectedIndexChanged
        'When combobox cbChannelConfig changes, update array ChannelConfig accordingly
        'as a function of the enabled channel
        Select Case True
            Case rbAin1.Checked
                ChannelConfig(0) = cbAnalogConfig.Text
            Case rbAin2.Checked
                ChannelConfig(1) = cbAnalogConfig.Text
            Case rbAin3.Checked
                ChannelConfig(2) = cbAnalogConfig.Text
            Case rbAin4.Checked
                ChannelConfig(3) = cbAnalogConfig.Text
        End Select
    End Sub

    Private Sub rbAin1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin1.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_1.Visible = False
        'diplay the configuration of analog channel 1 in combo box cbChannelConfig
        If rbAin1.Checked Then
            cbAnalogConfig.Text = ChannelConfig(0)
        End If
    End Sub

    Private Sub rbAin2_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin2.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_1.Visible = False
        'diplay the configuration of analog channel 2 in combo box cbChannelConfig
        If rbAin2.Checked Then
            cbAnalogConfig.Text = ChannelConfig(1)
        End If
    End Sub

    Private Sub rbAin3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin3.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_1.Visible = False
        'diplay the configuration of analog channel 3 in combo box cbChannelConfig
        If rbAin3.Checked Then
            cbAnalogConfig.Text = ChannelConfig(2)
        End If
    End Sub

    Private Sub rbAin4_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbAin4.CheckedChanged
        'display the proper combo box
        cbAnalogConfig.Visible = True
        cbDin0_1.Visible = False
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        If rbAin4.Checked Then
            cbAnalogConfig.Text = ChannelConfig(3)
        End If
    End Sub

    Private Sub btnClear_Click(sender As System.Object, e As System.EventArgs) Handles btnClear.Click
        'Clear the output list box
        lbOutput.Items.Clear()
        lblRealTime.Text = ""
    End Sub

    Private Sub cbDin0_1_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbDin0_1.SelectedIndexChanged
        'When combobox cbDin0_3 changes, update array ChannelConfig accordingly
        ChannelConfig(8) = cbDin0_1.Text
    End Sub

    Private Sub rbDin0_1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbDin0_1.CheckedChanged
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_1.Visible = True
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        cbDin0_1.Text = ChannelConfig(8)
    End Sub

    Private Sub rbDin2_CheckedChanged(sender As System.Object, e As System.EventArgs)
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_1.Visible = False
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
    End Sub

    Private Sub rbDin3_CheckedChanged(sender As System.Object, e As System.EventArgs)
        'display the proper combo boxes
        cbAnalogConfig.Visible = False
        cbDin0_1.Visible = False
    End Sub

End Class
